package com.lqkj.link.module.audit.controller;

import com.lqkj.link.message.MessageBean;
import com.lqkj.link.module.audit.service.AuditRecordService;
import com.lqkj.link.module.jwt.service.JwtService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

import static com.lqkj.link.APIVersion.VERSION_V1;

@RestController
@RequestMapping("/audit")
@Tag(name = "审核记录管理", description = "审核记录管理")
public class AuditRecordController {

    private final AuditRecordService auditRecordService;
    private final JwtService jwtService;

    public AuditRecordController(AuditRecordService auditRecordService, JwtService jwtService) {
        this.auditRecordService = auditRecordService;
        this.jwtService = jwtService;
    }

    @Operation(
            summary = "5.1.3.9 审核记录分页接口",
            description = "5.1.3.9 审核记录分页接口",
            parameters = {
                    @Parameter(name = "auditor", description = "审核员"),
                    @Parameter(name = "author", description = "作者"),
                    @Parameter(name = "authStatus", description = "审核状态，1待审核，2通过，3拒绝，4删除，5撤销"),
                    @Parameter(name = "page", description = "页码"),
                    @Parameter(name = "pageSize", description = "每页数据条数")
            }
    )
    @PostMapping("/manage/pageQuery")
    public MessageBean<Page<Map<String, Object>>> pageQuery(@RequestParam(required = false, defaultValue = "") String auditor,
                                                             @RequestParam(required = false, defaultValue = "") String author,
                                                             @RequestParam(required = false, defaultValue = "-1") Integer authStatus,
                                                             @RequestParam(required = false, defaultValue = "0") Integer page,
                                                             @RequestParam(required = false, defaultValue = "10") Integer pageSize) {
        return MessageBean.ok(auditRecordService.pageQuery(auditor, author, authStatus, page, pageSize), "审核记录分页接口");
    }

    @Operation(
            summary = "5.1.3.10 审核记录删除接口",
            description = "5.1.3.10 审核记录删除接口",
            parameters = {
                    @Parameter(name = "recordIds", required = true, description = "审核记录ID数组")
            }
    )
    @PostMapping("/manage/delete")
    public MessageBean delete(@RequestParam List<Integer> recordIds) {
        String delete = auditRecordService.delete(recordIds);
        return MessageBean.ok(null, delete);
    }

    @Operation(
            summary = "5.1.2.10 审核状态修改接口",
            description = "5.1.2.10 审核状态修改接口",
            parameters = {
                    @Parameter(name = "recordId", description = "审核记录ID", required = true),
                    @Parameter(name = "authStatus", description = "状态码，2通过，3拒绝，4删除", required = true)
            }
    )
    @PostMapping("/" + VERSION_V1 + "/audit")
    public MessageBean updateAuditStatus(@RequestParam Integer recordId,
                                         @RequestParam Integer authStatus,
                                         HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        String msg = auditRecordService.updateAuthStatus(recordId, authStatus, userCode);
        if (StringUtils.isNotBlank(msg)) return MessageBean.error(msg);
        return MessageBean.ok(null, "审核中心审核接口");
    }

    @Operation(
            summary = "5.1.2.26 申请/撤销审核",
            description = "5.1.2.26 申请/撤销审核",
            parameters = {
                    @Parameter(name = "zoneId", description = "作品/区域ID", required = true),
                    @Parameter(name = "isApply", description = "是否是提交，true提交，false撤销", required = true)
            }
    )
    @PostMapping("/" + VERSION_V1 + "/applyOrRevoke")
    public MessageBean<String> applyOrRevoke(@RequestParam Integer zoneId,
                                     @RequestParam Boolean isApply) {
        String msg = auditRecordService.applyOrRevoke(zoneId, isApply);
        if (StringUtils.isNotBlank(msg)) return MessageBean.error(msg);
        return MessageBean.ok(null, "申请/撤销审核");
    }
}
