package com.lqkj.link.module.authority.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.lqkj.link.config.LimitRequest.LimitRequest;
import com.lqkj.link.message.MessageBean;
import com.lqkj.link.module.authority.domain.UserInfo;
import com.lqkj.link.module.authority.service.UserInfoService;
import com.lqkj.link.module.base.service.BaseService;
import com.lqkj.link.module.jwt.service.JwtService;
import com.lqkj.link.util.AESUtils;
import com.lqkj.link.util.RSAUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.lqkj.link.APIVersion.VERSION_V1;

@RestController
@RequestMapping("/user")
@Tag(name = "用户管理", description = "用户管理")
@LimitRequest
public class UserInfoController {
    private final UserInfoService userInfoService;
    private final JwtService jwtService;
    private final BaseService baseService;

    public UserInfoController(UserInfoService userInfoService, JwtService jwtService, BaseService baseService) {
        this.userInfoService = userInfoService;
        this.jwtService = jwtService;
        this.baseService = baseService;
    }

    @Operation(
            summary = "5.1.1.3 获取登录用户接口",
            description = "5.1.1.3 获取登录用户接口"
    )
    @PostMapping("/" + VERSION_V1 + "/detail")
    public MessageBean<UserInfo> detail(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String username = jwtService.decryptUsernameWithHeader(authHeader);
        return MessageBean.ok(userInfoService.detailByUserCode(username), "获取登录用户");
    }

    @Operation(
            summary = "5.1.3.24 获取管理菜单接口",
            description = "5.1.3.24 获取管理菜单接口"
    )
    @PostMapping("/authList")
    public MessageBean authList(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String username = jwtService.decryptUsernameWithHeader(authHeader);
        return MessageBean.ok(userInfoService.findAuthListWithUserCode(username), "获取管理菜单接口");
    }

    @Operation(
            summary = "5.1.3.11 用户分页接口",
            description = "5.1.3.11 用户分页接口",
            parameters = {
                    @Parameter(name = "userCode", description = "用户账号"),
                    @Parameter(name = "displayName", description = "用户名"),
                    @Parameter(name = "page", description = "页码"),
                    @Parameter(name = "pageSize", description = "每页数据条数"),
                    @Parameter(name = "key", description = "RSA加密后的aesKey")
            }
    )
    @PostMapping("/manage/pageQuery")
    public MessageBean<JSONObject> pageQuery(@RequestParam(required = false, defaultValue = "") String userCode,
                                                 @RequestParam(required = false, defaultValue = "") String displayName,
                                                 @RequestParam(required = false, defaultValue = "0") Integer page,
                                                 @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                 @RequestParam String key) throws Exception {
        Page<UserInfo> userInfoPage = userInfoService.pageQuery(userCode, displayName, page, pageSize);
        String aesKey = RSAUtils.decryptBase64(key);
        List<UserInfo> list = new ArrayList<>();
        for (UserInfo userInfo : userInfoPage.getContent()) {
            list.add(AESUtils.encryptUser(userInfo, aesKey));
        }
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(userInfoPage));
        jsonObject.put("content", list);
        return MessageBean.ok(jsonObject, "用户分页接口");
    }

    @Operation(
            summary = "5.1.3.28 用户详情接口",
            description = "5.1.3.28 用户详情接口",
            parameters = {
                    @Parameter(name = "userId", required = true, description = "用户ID"),
                    @Parameter(name = "key", description = "RSA加密后的aesKey")
            }
    )
    @PostMapping("/manage/detail")
    public MessageBean<UserInfo> detailById(@RequestParam Integer userId,
                                            @RequestParam String key) throws Exception {
        String aesKey = RSAUtils.decryptBase64(key);
        return MessageBean.ok(AESUtils.encryptUser(userInfoService.detail(userId), aesKey), "用户详情接口");
    }

    @Operation(
            summary = "5.1.3.12 保存用户接口",
            description = "5.1.3.12 保存用户接口",
            requestBody = @io.swagger.v3.oas.annotations.parameters.RequestBody(
                    description = "用户实体",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = UserInfo.class)
                    )
            )
    )
    @PostMapping("/manage/save")
    public MessageBean save(@RequestBody UserInfo userInfo) {
        userInfo.setUserCode(RSAUtils.decryptBase64(userInfo.getUserCode()));
        if (StringUtils.isNotBlank(userInfo.getPassword()) && userInfo.getPassword().length() == 172) {
            userInfo.setPassword(RSAUtils.decryptBase64(userInfo.getPassword()));
        }
//        userInfo.setAuthorizationCode(RSAUtils.decryptBase64(userInfo.getAuthorizationCode()));
        String message = userInfoService.save(userInfo);
        if (message == null) return MessageBean.ok(null, "保存用户接口");
        return MessageBean.error(message);
    }

    @Operation(
            summary = "5.1.3.13 删除用户接口",
            description = "5.1.3.13 删除用户接口",
            parameters = {
                    @Parameter(name = "userIds", required = true, description = "用户ID列表")
            }
    )
    @PostMapping("/manage/delete")
    public MessageBean delete(@RequestParam List<Integer> userIds) {
        userInfoService.delete(userIds);
        return MessageBean.ok(null, "删除用户接口");
    }

    @Operation(
            summary = "5.1.2.12 上传头像",
            description = "5.1.2.12 上传头像",
            parameters = {
                    @Parameter(name = "file", description = "图标文件", required = true)
            }
    )
    @PostMapping("/" + VERSION_V1 + "/uploadHead")
    public MessageBean<String> uploadImg(@RequestParam MultipartFile file) {
        return baseService.uploadImg(file, "user/head/");
    }

    @Operation(
            summary = "5.1.2.13 用户设置",
            description = "5.1.2.13 用户设置",
            parameters = {
                    @Parameter(name = "displayName", description = "用户名称"),
                    @Parameter(name = "headImgPath", description = "头像地址")
            }
    )
    @PostMapping("/" + VERSION_V1 + "/updateInfo")
    public MessageBean<String> saveUser(@RequestParam(required = false) String displayName,
                                         @RequestParam(required = false) String headImgPath,
                                         HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        userInfoService.updateInfo(userCode, displayName, headImgPath);
        return MessageBean.ok(null, "更新用户信息");
    }

    @Operation(
            summary = "5.1.2.38 检查是否需要刷新资源",
            description = "5.1.2.38 检查是否需要刷新资源"
    )
    @PostMapping("/" + VERSION_V1 + "/checkRefreshResource")
    public MessageBean<Boolean> checkRefreshResource(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        return MessageBean.ok(userInfoService.checkRefreshResource(userCode), "检查是否需要刷新资源");
    }

    @Operation(
            summary = "5.1.2.39 更新用户刷新资源状态",
            description = "5.1.2.39 更新用户刷新资源状态"
    )
    @PostMapping("/" + VERSION_V1 + "/resourceRefreshed")
    public MessageBean<String> resourceRefreshed(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        userInfoService.resourceRefreshed(userCode);
        return MessageBean.ok(null, "更新用户刷新资源状态");
    }

    @Operation(
            summary = "注册用户接口",
            description = "注册用户接口",
            requestBody = @io.swagger.v3.oas.annotations.parameters.RequestBody(
                    description = "用户实体",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = UserInfo.class)
                    )
            )
    )
    @PostMapping("/manage/register")
    public MessageBean register(@RequestBody UserInfo userInfo,
                                @RequestParam(name = "captcha") String captcha,
                                HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = null;
        if (Objects.nonNull(authHeader)){
             userCode = jwtService.decryptUsernameWithHeader(authHeader);
        }
        userInfoService.register(userInfo, false, userCode, captcha);
        return MessageBean.ok(null, "注册用户接口");
    }


    /**
     * 保存用户ue设置
     * @param autosaveTime
     * @param movingSpeed
     * @param request
     * @return
     */
    @PostMapping("/saveSettings")
    public MessageBean<String> saveSettings(@RequestParam Integer autosaveTime,
                                            @RequestParam Integer movingSpeed,
                                            HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        userInfoService.saveSettings(autosaveTime, movingSpeed, userCode);
        return MessageBean.ok(null, "保存用户设置");
    }

    /**
     * 发送短信验证码
     * @param phoneNumber
     * @return
     */
    @LimitRequest(time = 60 * 1000)
    @PostMapping("/sendSms")
    public MessageBean<String> sendSms(@RequestParam String phoneNumber) {
        userInfoService.sendSms(phoneNumber);
        return MessageBean.ok(null, "发送短信成功");
    }
}
